<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Exception;
use Service\Integration\IntegrationSM;
use Teknisa\Libs\Util\Masks\Picture;
use Teknisa\Model\Teknisa\Entities\CpCall;
use Teknisa\Model\Teknisa\Entities\Orcasolicita;
use Teknisa\Model\Teknisa\Entities\Parcsoliserv;
use Teknisa\Model\Teknisa\Entities\Tarefa;
use Util\Instance\Repository;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Util\Messages;

class Budget
{
    /**
     * @var EntityManager
     */
    public $entityManager;
    /**
     * @var Utilities
     */
    public $environment;
    /**
     * @var int|mixed
     */
    private $nrorg;

    public function __construct()
    {
        $this->entityManager  = InstanceProvider::getEntityManager();
        $this->environment    = InstanceProvider::getEnvironment();

        $this->nrorg = $this->environment->getCurrentOrganizationId();
    }

    public function budgetItem() {
        $budgetItems = InstanceProvider::getRepository()->getBudgetItem()->budgetItem(InstanceProvider::getEnvironment()->getCurrentOrganizationId(), null);
        return $budgetItems;
    }

    public function classificationBudgetItem($classificationId) {
        $budgetItems = InstanceProvider::getRepository()->getBudgetItem()->classificationBudgetItem($classificationId);
        return $budgetItems;
    }

    public function budgetItemSave($id,$name,$valueItem,$unitMeasure,$status) {
        $connection = $this->entityManager->getConnection();
        try {
            $this->entityManager->beginTransaction();
            $entityManager = InstanceProvider::getEntityManager();
            $budgetItem = null;
            if($id){
                /** @var Entities\BudgetItem $budgetItem */
                $budgetItem = InstanceProvider::getRepository()->getBudgetItem()->find($id);
            }
            $valueItem = Utilities::fixBugSaveCurrencyMask($valueItem);
            $budgetItem = InstanceProvider::getFactory()->getBudgetItem()->save($budgetItem,$name, $valueItem, $unitMeasure,$status);
            $entityManager->persist($budgetItem);
            $entityManager->flush();
            $connection->commit();
        }  catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function budgetItemRemove($id)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $entityManager = InstanceProvider::getEntityManager();
            /** @var Entities\BudgetItem $budgetItem */
            $budgetItem = InstanceProvider::getRepository()->getBudgetItem()->find($id);
            $budgetItemBudget = InstanceProvider::getRepository()->getBudgetItemBudget()->findOneBy(array('budgetItem' => $budgetItem));
    
            if(!$budgetItemBudget){
                $classificationBudgetItems = InstanceProvider::getRepository()->getClassificationBudgetItem()
                    ->findBy(array('budgetItem' => $budgetItem));
    
                foreach ($classificationBudgetItems as $classificationBudgetItem) {
                    $entityManager->remove($classificationBudgetItem);
                }
                $entityManager->flush();
    
                $budgetItem->deleteEntity();
            }else{
                $budgetItem->setStatus(Entities\BudgetItem::IS_NOT_ACTIVE);
            }
            $entityManager->persist($budgetItem);
            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function addBudgetItemInClassificationSave($budgetItemId,$classificationId) {
        $connection = $this->entityManager->getConnection();
        try { 
            $connection->beginTransaction();
            $entityManager = InstanceProvider::getEntityManager();
            if($budgetItemId && $classificationId){
                /** @var Entities\BudgetItem $budgetItem */
                $budgetItem = InstanceProvider::getRepository()->getBudgetItem()->find($budgetItemId);
                /** @var Entities\Classification $classification */
                $classification = InstanceProvider::getRepository()->getClassification()->find($classificationId);
                $classificationBudgetItem = InstanceProvider::getRepository()->getClassificationBudgetItem()
                    ->findOneBy(array('classification' => $classificationId,'budgetItem' => $budgetItemId,));
                if(!$classificationBudgetItem){
                    $classificationBudgetItem = InstanceProvider::getFactory()->getclassificationBudgetItem()->create($budgetItem,$classification);
    
                    $entityManager->persist($classificationBudgetItem);
                    $entityManager->flush();
                    $connection->commit();
                }
            }
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function classificationBudgetItemRemove($id) {
        $connection = $this->entityManager->getConnection();
        try {  
            $connection->beginTransaction();
            $entityManager = InstanceProvider::getEntityManager();
            /** @var Entities\ClassificationBudgetItem $budgetItem */
            $classificationBudgetItem = InstanceProvider::getRepository()->getClassificationBudgetItem()->find($id);
            $entityManager->remove($classificationBudgetItem);
            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
}
